#include "stdafx.h"
#include "juce.h"

#include <iostream>
using namespace std;

class MyMidiInputCallback : public MidiInputCallback
{
public:
	MyMidiInputCallback(MidiOutput* midiOut)
		: midiOut(midiOut)
	{}
	void handleIncomingMidiMessage (MidiInput *source, const MidiMessage &message)
	{
		midiOut->sendMessageNow(message);
		if(message.isNoteOn() || message.isNoteOff())
		{
			// auf nchsten Kanal, transponiert
			MidiMessage m1(message);
			m1.setChannel(m1.getChannel()+1); // To Do: Kanal <= 15 ?
			m1.setNoteNumber(m1.getNoteNumber()+7); // To Do: Note <= 127 ?
			midiOut->sendMessageNow(m1);

			// verzgert auf bernchsten Kanal
			MidiMessage m2(message);
			m2.setChannel(m2.getChannel()+2); // To Do: Kanal <= 15 ?
			buffer.addEvent(m2, 0);
			midiOut->sendBlockOfMessages(buffer, Time::getMillisecondCounter() + 500.0, 1000.0);
			buffer.clear();
		}
	}
private:
	MidiOutput* midiOut;
	MidiBuffer buffer;
};

int _tmain(int argc, _TCHAR* argv[])
{
	// Ausgabe initialisieren

	StringArray outputDevices = MidiOutput::getDevices();
	for(int i = 0; i < outputDevices.size(); i++)
	{
		cout << outputDevices[i] << endl;
	}

	MidiOutput* midiOut = MidiOutput::openDevice(0);

	// Ausgabe einzelner Noten "zu Fu"

	midiOut->sendMessageNow(MidiMessage::noteOn(0, 60, uint8(100)));
	Thread::sleep(300);
	midiOut->sendMessageNow(MidiMessage::noteOff(0, 60));

	midiOut->sendMessageNow(MidiMessage::noteOn(0, 64, uint8(100)));
	Thread::sleep(300);
	midiOut->sendMessageNow(MidiMessage::noteOff(0, 64));

	midiOut->sendMessageNow(MidiMessage::noteOn(0, 67, uint8(100)));
	Thread::sleep(300);
	midiOut->sendMessageNow(MidiMessage::noteOff(0, 67));

	Thread::sleep(100);

	// Ausgabe mit MidiBuffer-Objekt

	MidiBuffer buffer;
	buffer.addEvent(MidiMessage::noteOn(0, 60, uint8(100)), 0);
	buffer.addEvent(MidiMessage::noteOff(0, 60), 300);
	buffer.addEvent(MidiMessage::noteOn(0, 64, uint8(100)), 300);
	buffer.addEvent(MidiMessage::noteOff(0, 64), 600);
	buffer.addEvent(MidiMessage::noteOn(0, 67, uint8(100)), 600);
	buffer.addEvent(MidiMessage::noteOff(0, 67), 900);

	double startMilliseconds = Time::getMillisecondCounter() + 500.0;
	midiOut->startBackgroundThread();
	midiOut->sendBlockOfMessages(buffer, startMilliseconds, 1000.0);
	midiOut->sendBlockOfMessages(buffer, startMilliseconds + 300.0, 1000.0);
	Thread::sleep(2000); // Verarbeitung luft im Hintergrund

	// Eingabe und Verarbeitung

	StringArray inputDevices = MidiInput::getDevices();
	for(int i = 0; i < inputDevices.size(); i++)
	{
		cout << inputDevices[i] << endl;
	}

	MyMidiInputCallback inputCallback(midiOut);
	MidiInput* midiIn = MidiInput::openDevice(0, &inputCallback);
	midiIn->start();
	while(!KeyPress::isKeyCurrentlyDown(32))
	{
		Thread::sleep(100); // Verarbeitung luft im Hintergrund
	}
	midiIn->stop();

	midiOut->stopBackgroundThread();

	delete midiIn;
	delete midiOut;

	return 0;
}

